@@ -1,7 +1,7 @@
 /*
  * aconf-dj.h
  *
- * Copyright 2002 Glyph & Cog, LLC
+ * Copyright 2002-2003 Glyph & Cog, LLC
  */
 
 #ifndef ACONF_H
@@ -22,11 +22,6 @@
  */
 #undef OPI_SUPPORT
 
-/*
- * Use gzip instead of uncompress.
- */
-#define USE_GZIP 1
-
 /*
  * Directory with the Xpdf app-defaults file.
  */
@@ -51,6 +46,7 @@
 #define HAVE_POPEN 1
 #undef HAVE_MKSTEMP
 #undef SELECT_TAKES_INT
+#define HAVE_STD_SORT 1
 #undef HAVE_FSEEK64
 
 /*
@@ -58,11 +54,6 @@
  */
 #undef HAVE_X11_XPM_H
 
-/*
- * This is defined if using t1lib.
- */
-#undef HAVE_T1LIB_H
-
 /*
  * One of these is defined if using FreeType (version 1 or 2).
  */
