@@ -6,7 +6,7 @@
  * file because otherwise the configure script will munge any
  * #define/#undef constructs.
  *
- * Copyright 2002 Glyph & Cog, LLC
+ * Copyright 2002-2003 Glyph & Cog, LLC
  */
 
 #ifndef ACONF2_H
@@ -19,6 +19,9 @@
 #define USE_GCC_PRAGMAS
 #endif
 /* There is a bug in the version of gcc which ships with MacOS X 10.2 */
+#if defined(__APPLE__) && defined(__MACH__)
+#  include <AvailabilityMacros.h>
+#endif
 #ifdef MAC_OS_X_VERSION_MAX_ALLOWED
 #  if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2
 #    undef USE_GCC_PRAGMAS
