@@ -2,15 +2,16 @@
 //
 // GHash.cc
 //
-// Copyright 2001-2002 Glyph & Cog, LLC
+// Copyright 2001-2003 Glyph & Cog, LLC
 //
 //========================================================================
 
-#ifdef __GNUC__
+#include <aconf.h>
+
+#ifdef USE_GCC_PRAGMAS
 #pragma implementation
 #endif
 
-#include <aconf.h>
 #include "gmem.h"
 #include "GString.h"
 #include "GHash.h"
@@ -19,7 +20,10 @@
 
 struct GHashBucket {
   GString *key;
-  void *val;
+  union {
+    void *p;
+    int i;
+  } val;
   GHashBucket *next;
 };
 
@@ -35,7 +39,7 @@ GHash::GHash(GBool deleteKeysA) {
 
   deleteKeys = deleteKeysA;
   size = 7;
-  tab = (GHashBucket **)gmalloc(size * sizeof(GHashBucket *));
+  tab = (GHashBucket **)gmallocn(size, sizeof(GHashBucket *));
   for (h = 0; h < size; ++h) {
     tab[h] = NULL;
   }
@@ -60,41 +64,71 @@ GHash::~GHash() {
 }
 
 void GHash::add(GString *key, void *val) {
-  GHashBucket **oldTab;
   GHashBucket *p;
-  int oldSize, i, h;
+  int h;
 
   // expand the table if necessary
   if (len >= size) {
-    oldSize = size;
-    oldTab = tab;
-    size = 2*size + 1;
-    tab = (GHashBucket **)gmalloc(size * sizeof(GHashBucket *));
-    for (h = 0; h < size; ++h) {
-      tab[h] = NULL;
-    }
-    for (i = 0; i < oldSize; ++i) {
-      while (oldTab[i]) {
-	p = oldTab[i];
-	oldTab[i] = oldTab[i]->next;
-	h = hash(p->key);
-	p->next = tab[h];
-	tab[h] = p;
-      }
-    }
-    gfree(oldTab);
+    expand();
   }
 
   // add the new symbol
   p = new GHashBucket;
   p->key = key;
-  p->val = val;
+  p->val.p = val;
   h = hash(key);
   p->next = tab[h];
   tab[h] = p;
   ++len;
 }
 
+void GHash::add(GString *key, int val) {
+  GHashBucket *p;
+  int h;
+
+  // expand the table if necessary
+  if (len >= size) {
+    expand();
+  }
+
+  // add the new symbol
+  p = new GHashBucket;
+  p->key = key;
+  p->val.i = val;
+  h = hash(key);
+  p->next = tab[h];
+  tab[h] = p;
+  ++len;
+}
+
+void GHash::replace(GString *key, void *val) {
+  GHashBucket *p;
+  int h;
+
+  if ((p = find(key, &h))) {
+    p->val.p = val;
+    if (deleteKeys) {
+      delete key;
+    }
+  } else {
+    add(key, val);
+  }
+}
+
+void GHash::replace(GString *key, int val) {
+  GHashBucket *p;
+  int h;
+
+  if ((p = find(key, &h))) {
+    p->val.i = val;
+    if (deleteKeys) {
+      delete key;
+    }
+  } else {
+    add(key, val);
+  }
+}
+
 void *GHash::lookup(GString *key) {
   GHashBucket *p;
   int h;
@@ -102,17 +136,37 @@ void *GHash::lookup(GString *key) {
   if (!(p = find(key, &h))) {
     return NULL;
   }
-  return p->val;
+  return p->val.p;
 }
 
-void *GHash::lookup(char *key) {
+int GHash::lookupInt(GString *key) {
+  GHashBucket *p;
+  int h;
+
+  if (!(p = find(key, &h))) {
+    return 0;
+  }
+  return p->val.i;
+}
+
+void *GHash::lookup(const char *key) {
   GHashBucket *p;
   int h;
 
   if (!(p = find(key, &h))) {
     return NULL;
   }
-  return p->val;
+  return p->val.p;
+}
+
+int GHash::lookupInt(const char *key) {
+  GHashBucket *p;
+  int h;
+
+  if (!(p = find(key, &h))) {
+    return 0;
+  }
+  return p->val.i;
 }
 
 void *GHash::remove(GString *key) {
@@ -132,13 +186,36 @@ void *GHash::remove(GString *key) {
   if (deleteKeys) {
     delete p->key;
   }
-  val = p->val;
+  val = p->val.p;
   delete p;
   --len;
   return val;
 }
 
-void *GHash::remove(char *key) {
+int GHash::removeInt(GString *key) {
+  GHashBucket *p;
+  GHashBucket **q;
+  int val;
+  int h;
+
+  if (!(p = find(key, &h))) {
+    return 0;
+  }
+  q = &tab[h];
+  while (*q != p) {
+    q = &((*q)->next);
+  }
+  *q = p->next;
+  if (deleteKeys) {
+    delete p->key;
+  }
+  val = p->val.i;
+  delete p;
+  --len;
+  return val;
+}
+
+void *GHash::remove(const char *key) {
   GHashBucket *p;
   GHashBucket **q;
   void *val;
@@ -155,7 +232,30 @@ void *GHash::remove(char *key) {
   if (deleteKeys) {
     delete p->key;
   }
-  val = p->val;
+  val = p->val.p;
+  delete p;
+  --len;
+  return val;
+}
+
+int GHash::removeInt(const char *key) {
+  GHashBucket *p;
+  GHashBucket **q;
+  int val;
+  int h;
+
+  if (!(p = find(key, &h))) {
+    return 0;
+  }
+  q = &tab[h];
+  while (*q != p) {
+    q = &((*q)->next);
+  }
+  *q = p->next;
+  if (deleteKeys) {
+    delete p->key;
+  }
+  val = p->val.i;
   delete p;
   --len;
   return val;
@@ -183,7 +283,27 @@ GBool GHash::getNext(GHashIter **iter, GString **key, void **val) {
     (*iter)->p = tab[(*iter)->h];
   }
   *key = (*iter)->p->key;
-  *val = (*iter)->p->val;
+  *val = (*iter)->p->val.p;
+  return gTrue;
+}
+
+GBool GHash::getNext(GHashIter **iter, GString **key, int *val) {
+  if (!*iter) {
+    return gFalse;
+  }
+  if ((*iter)->p) {
+    (*iter)->p = (*iter)->p->next;
+  }
+  while (!(*iter)->p) {
+    if (++(*iter)->h == size) {
+      delete *iter;
+      *iter = NULL;
+      return gFalse;
+    }
+    (*iter)->p = tab[(*iter)->h];
+  }
+  *key = (*iter)->p->key;
+  *val = (*iter)->p->val.i;
   return gTrue;
 }
 
@@ -192,6 +312,30 @@ void GHash::killIter(GHashIter **iter) {
   *iter = NULL;
 }
 
+void GHash::expand() {
+  GHashBucket **oldTab;
+  GHashBucket *p;
+  int oldSize, h, i;
+
+  oldSize = size;
+  oldTab = tab;
+  size = 2*size + 1;
+  tab = (GHashBucket **)gmallocn(size, sizeof(GHashBucket *));
+  for (h = 0; h < size; ++h) {
+    tab[h] = NULL;
+  }
+  for (i = 0; i < oldSize; ++i) {
+    while (oldTab[i]) {
+      p = oldTab[i];
+      oldTab[i] = oldTab[i]->next;
+      h = hash(p->key);
+      p->next = tab[h];
+      tab[h] = p;
+    }
+  }
+  gfree(oldTab);
+}
+
 GHashBucket *GHash::find(GString *key, int *h) {
   GHashBucket *p;
 
@@ -204,7 +348,7 @@ GHashBucket *GHash::find(GString *key, int *h) {
   return NULL;
 }
 
-GHashBucket *GHash::find(char *key, int *h) {
+GHashBucket *GHash::find(const char *key, int *h) {
   GHashBucket *p;
 
   *h = hash(key);
@@ -217,7 +361,7 @@ GHashBucket *GHash::find(char *key, int *h) {
 }
 
 int GHash::hash(GString *key) {
-  char *p;
+  const char *p;
   unsigned int h;
   int i;
 
@@ -228,8 +372,8 @@ int GHash::hash(GString *key) {
   return (int)(h % size);
 }
 
-int GHash::hash(char *key) {
-  char *p;
+int GHash::hash(const char *key) {
+  const char *p;
   unsigned int h;
 
   h = 0;
