@@ -2,15 +2,17 @@
 //
 // GList.cc
 //
-// Copyright 2001-2002 Glyph & Cog, LLC
+// Copyright 2001-2003 Glyph & Cog, LLC
 //
 //========================================================================
 
-#ifdef __GNUC__
+#include <aconf.h>
+
+#ifdef USE_GCC_PRAGMAS
 #pragma implementation
 #endif
 
-#include <aconf.h>
+#include <stdlib.h>
 #include <string.h>
 #include "gmem.h"
 #include "GList.h"
@@ -21,14 +23,14 @@
 
 GList::GList() {
   size = 8;
-  data = (void **)gmalloc(size * sizeof(void*));
+  data = (void **)gmallocn(size, sizeof(void*));
   length = 0;
   inc = 0;
 }
 
 GList::GList(int sizeA) {
-  size = sizeA;
-  data = (void **)gmalloc(size * sizeof(void*));
+  size = sizeA ? sizeA : 8;
+  data = (void **)gmallocn(size, sizeof(void*));
   length = 0;
   inc = 0;
 }
@@ -37,6 +39,16 @@ GList::~GList() {
   gfree(data);
 }
 
+GList *GList::copy() {
+  GList *ret;
+
+  ret = new GList(length);
+  ret->length = length;
+  memcpy(ret->data, data, length * sizeof(void *));
+  ret->inc = inc;
+  return ret;
+}
+
 void GList::append(void *p) {
   if (length >= size) {
     expand();
@@ -59,6 +71,9 @@ void GList::insert(int i, void *p) {
   if (length >= size) {
     expand();
   }
+  if (i < 0) {
+    i = 0;
+  }
   if (i < length) {
     memmove(data+i+1, data+i, (length - i) * sizeof(void *));
   }
@@ -80,12 +95,28 @@ void *GList::del(int i) {
   return p;
 }
 
+void GList::sort(int (*cmp)(const void *obj1, const void *obj2)) {
+  qsort(data, length, sizeof(void *), cmp);
+}
+
+void GList::reverse() {
+  void *t;
+  int n, i;
+
+  n = length / 2;
+  for (i = 0; i < n; ++i) {
+    t = data[i];
+    data[i] = data[length - 1 - i];
+    data[length - 1 - i] = t;
+  }
+}
+
 void GList::expand() {
   size += (inc > 0) ? inc : size;
-  data = (void **)grealloc(data, size * sizeof(void*));
+  data = (void **)greallocn(data, size, sizeof(void*));
 }
 
 void GList::shrink() {
   size -= (inc > 0) ? inc : size/2;
-  data = (void **)grealloc(data, size * sizeof(void*));
+  data = (void **)greallocn(data, size, sizeof(void*));
 }
