@@ -4,19 +4,16 @@
 //
 // Miscellaneous file and directory name manipulation.
 //
-// Copyright 1996-2002 Glyph & Cog, LLC
+// Copyright 1996-2003 Glyph & Cog, LLC
 //
 //========================================================================
 
 #include <aconf.h>
 
-#ifdef WIN32
-   extern "C" {
-#  ifndef _MSC_VER
-#    include <kpathsea/win32lib.h>
-#  endif
-   }
-#else // !WIN32
+#ifdef _WIN32
+#  include <time.h>
+#  include <direct.h>
+#else
 #  if defined(MACOS)
 #    include <sys/stat.h>
 #  elif !defined(ACORN)
@@ -24,6 +21,7 @@
 #    include <sys/stat.h>
 #    include <fcntl.h>
 #  endif
+#  include <time.h>
 #  include <limits.h>
 #  include <string.h>
 #  if !defined(VMS) && !defined(ACORN) && !defined(MACOS)
@@ -32,7 +30,7 @@
 #  if defined(VMS) && (__DECCXX_VER < 50200000)
 #    include <unixlib.h>
 #  endif
-#endif // WIN32
+#endif // _WIN32
 #include "GString.h"
 #include "gfile.h"
 
@@ -49,7 +47,7 @@ GString *getHomeDir() {
   //---------- VMS ----------
   return new GString("SYS$LOGIN:");
 
-#elif defined(__EMX__) || defined(WIN32)
+#elif defined(__EMX__) || defined(_WIN32)
   //---------- OS/2+EMX and Win32 ----------
   char *s;
   GString *ret;
@@ -95,8 +93,8 @@ GString *getCurrentDir() {
 
 #if defined(__EMX__)
   if (_getcwd2(buf, sizeof(buf)))
-#elif defined(WIN32)
-  if (GetCurrentDirectory(sizeof(buf), buf))
+#elif defined(_WIN32)
+  if (GetCurrentDirectoryA(sizeof(buf), buf))
 #elif defined(ACORN)
   if (strcpy(buf, "@"))
 #elif defined(MACOS)
@@ -108,7 +106,7 @@ GString *getCurrentDir() {
   return new GString();
 }
 
-GString *appendToPath(GString *path, char *fileName) {
+GString *appendToPath(GString *path, const char *fileName) {
 #if defined(VMS)
   //---------- VMS ----------
   //~ this should handle everything necessary for file
@@ -149,7 +147,7 @@ GString *appendToPath(GString *path, char *fileName) {
   }
   return path;
 
-#elif defined(WIN32)
+#elif defined(_WIN32)
   //---------- Win32 ----------
   GString *tmp;
   char buf[256];
@@ -158,7 +156,7 @@ GString *appendToPath(GString *path, char *fileName) {
   tmp = new GString(path);
   tmp->append('/');
   tmp->append(fileName);
-  GetFullPathName(tmp->getCString(), sizeof(buf), buf, &fp);
+  GetFullPathNameA(tmp->getCString(), sizeof(buf), buf, &fp);
   delete tmp;
   path->clear();
   path->append(buf);
@@ -285,16 +283,16 @@ GString *grabPath(char *fileName) {
     return new GString(fileName, p + 1 - fileName);
   return new GString();
 
-#elif defined(__EMX__) || defined(WIN32)
+#elif defined(__EMX__) || defined(_WIN32)
   //---------- OS/2+EMX and Win32 ----------
   char *p;
 
   if ((p = strrchr(fileName, '/')))
-    return new GString(fileName, p - fileName);
+    return new GString(fileName, (int)(p - fileName));
   if ((p = strrchr(fileName, '\\')))
-    return new GString(fileName, p - fileName);
+    return new GString(fileName, (int)(p - fileName));
   if ((p = strrchr(fileName, ':')))
-    return new GString(fileName, p + 1 - fileName);
+    return new GString(fileName, (int)(p + 1 - fileName));
   return new GString();
 
 #elif defined(ACORN)
@@ -329,7 +327,7 @@ GBool isAbsolutePath(char *path) {
   return strchr(path, ':') ||
 	 (path[0] == '[' && path[1] != '.' && path[1] != '-');
 
-#elif defined(__EMX__) || defined(WIN32)
+#elif defined(__EMX__) || defined(_WIN32)
   //---------- OS/2+EMX and Win32 ----------
   return path[0] == '/' || path[0] == '\\' || path[1] == ':';
 
@@ -359,13 +357,13 @@ GString *makePathAbsolute(GString *path) {
   }
   return path;
 
-#elif defined(WIN32)
+#elif defined(_WIN32)
   //---------- Win32 ----------
   char buf[_MAX_PATH];
   char *fp;
 
   buf[0] = '\0';
-  if (!GetFullPathName(path->getCString(), _MAX_PATH, buf, &fp)) {
+  if (!GetFullPathNameA(path->getCString(), _MAX_PATH, buf, &fp)) {
     path->clear();
     return path;
   }
@@ -430,7 +428,7 @@ GString *makePathAbsolute(GString *path) {
 }
 
 time_t getModTime(char *fileName) {
-#ifdef WIN32
+#ifdef _WIN32
   //~ should implement this, but it's (currently) only used in xpdf
   return 0;
 #else
@@ -443,26 +441,48 @@ time_t getModTime(char *fileName) {
 #endif
 }
 
-GBool openTempFile(GString **name, FILE **f, char *mode, char *ext) {
-#if defined(WIN32)
+GBool openTempFile(GString **name, FILE **f,
+		   const char *mode, const char *ext) {
+#if defined(_WIN32)
   //---------- Win32 ----------
-  char *s;
-  char buf[_MAX_PATH];
-  char *fp;
+  char *tempDir;
+  GString *s, *s2;
+  FILE *f2;
+  int t, i;
 
-  if (!(s = _tempnam(getenv("TEMP"), NULL))) {
-    return gFalse;
+  // this has the standard race condition problem, but I haven't found
+  // a better way to generate temp file names with extensions on
+  // Windows
+  if ((tempDir = getenv("TEMP"))) {
+    s = new GString(tempDir);
+    s->append('\\');
+  } else {
+    s = new GString();
   }
-  *name = new GString(s);
-  free(s);
-  if (ext) {
-    (*name)->append(ext);
+  s->appendf("x_{0:d}_{1:d}_",
+	     (int)GetCurrentProcessId(), (int)GetCurrentThreadId());
+  t = (int)time(NULL);
+  for (i = 0; i < 1000; ++i) {
+    s2 = s->copy()->appendf("{0:d}", t + i);
+    if (ext) {
+      s2->append(ext);
+    }
+    if (!(f2 = fopen(s2->getCString(), "r"))) {
+      if (!(f2 = fopen(s2->getCString(), mode))) {
+	delete s2;
+	delete s;
+	return gFalse;
+      }
+      *name = s2;
+      *f = f2;
+      delete s;
+      return gTrue;
+    }
+    fclose(f2);
+    delete s2;
   }
-  if (!(*f = fopen((*name)->getCString(), mode))) {
-    delete (*name);
-    return gFalse;
-  }
-  return gTrue;
+  delete s;
+  return gFalse;
 #elif defined(VMS) || defined(__EMX__) || defined(ACORN) || defined(MACOS)
   //---------- non-Unix ----------
   char *s;
@@ -480,6 +500,7 @@ GBool openTempFile(GString **name, FILE **f, char *mode, char *ext) {
   }
   if (!(*f = fopen((*name)->getCString(), mode))) {
     delete (*name);
+    *name = NULL;
     return gFalse;
   }
   return gTrue;
@@ -524,12 +545,21 @@ GBool openTempFile(GString **name, FILE **f, char *mode, char *ext) {
   }
   if (fd < 0 || !(*f = fdopen(fd, mode))) {
     delete *name;
+    *name = NULL;
     return gFalse;
   }
   return gTrue;
 #endif
 }
 
+GBool createDir(char *path, int mode) {
+#ifdef _WIN32
+  return !mkdir(path);
+#else
+  return !mkdir(path, mode);
+#endif
+}
+
 GBool executeCommand(char *cmd) {
 #ifdef VMS
   return system(cmd) ? gTrue : gFalse;
@@ -538,6 +568,107 @@ GBool executeCommand(char *cmd) {
 #endif
 }
 
+#ifdef _WIN32
+GString *fileNameToUTF8(char *path) {
+  GString *s;
+  char *p;
+
+  s = new GString();
+  for (p = path; *p; ++p) {
+    if (*p & 0x80) {
+      s->append((char)(0xc0 | ((*p >> 6) & 0x03)));
+      s->append((char)(0x80 | (*p & 0x3f)));
+    } else {
+      s->append(*p);
+    }
+  }
+  return s;
+}
+
+GString *fileNameToUTF8(wchar_t *path) {
+  GString *s;
+  wchar_t *p;
+
+  s = new GString();
+  for (p = path; *p; ++p) {
+    if (*p < 0x80) {
+      s->append((char)*p);
+    } else if (*p < 0x800) {
+      s->append((char)(0xc0 | ((*p >> 6) & 0x1f)));
+      s->append((char)(0x80 | (*p & 0x3f)));
+    } else {
+      s->append((char)(0xe0 | ((*p >> 12) & 0x0f)));
+      s->append((char)(0x80 | ((*p >> 6) & 0x3f)));
+      s->append((char)(0x80 | (*p & 0x3f)));
+    }
+  }
+  return s;
+}
+#endif
+
+FILE *openFile(const char *path, const char *mode) {
+#ifdef _WIN32
+  OSVERSIONINFO version;
+  wchar_t wPath[_MAX_PATH + 1];
+  char nPath[_MAX_PATH + 1];
+  wchar_t wMode[8];
+  const char *p;
+  int i;
+
+  // NB: _wfopen is only available in NT
+  version.dwOSVersionInfoSize = sizeof(version);
+  GetVersionEx(&version);
+  if (version.dwPlatformId == VER_PLATFORM_WIN32_NT) {
+    for (p = path, i = 0; *p && i < _MAX_PATH; ++i) {
+      if ((p[0] & 0xe0) == 0xc0 &&
+	  p[1] && (p[1] & 0xc0) == 0x80) {
+	wPath[i] = (wchar_t)(((p[0] & 0x1f) << 6) |
+			     (p[1] & 0x3f));
+	p += 2;
+      } else if ((p[0] & 0xf0) == 0xe0 &&
+		 p[1] && (p[1] & 0xc0) == 0x80 &&
+		 p[2] && (p[2] & 0xc0) == 0x80) {
+	wPath[i] = (wchar_t)(((p[0] & 0x0f) << 12) |
+			     ((p[1] & 0x3f) << 6) |
+			     (p[2] & 0x3f));
+	p += 3;
+      } else {
+	wPath[i] = (wchar_t)(p[0] & 0xff);
+	p += 1;
+      }
+    }
+    wPath[i] = (wchar_t)0;
+    for (i = 0; mode[i] && i < sizeof(mode) - 1; ++i) {
+      wMode[i] = (wchar_t)(mode[i] & 0xff);
+    }
+    wMode[i] = (wchar_t)0;
+    return _wfopen(wPath, wMode);
+  } else {
+    for (p = path, i = 0; *p && i < _MAX_PATH; ++i) {
+      if ((p[0] & 0xe0) == 0xc0 &&
+	  p[1] && (p[1] & 0xc0) == 0x80) {
+	nPath[i] = (char)(((p[0] & 0x1f) << 6) |
+			  (p[1] & 0x3f));
+	p += 2;
+      } else if ((p[0] & 0xf0) == 0xe0 &&
+		 p[1] && (p[1] & 0xc0) == 0x80 &&
+		 p[2] && (p[2] & 0xc0) == 0x80) {
+	nPath[i] = (char)(((p[1] & 0x3f) << 6) |
+			  (p[2] & 0x3f));
+	p += 3;
+      } else {
+	nPath[i] = p[0];
+	p += 1;
+      }
+    }
+    nPath[i] = '\0';
+    return fopen(nPath, mode);
+  }
+#else
+  return fopen(path, mode);
+#endif
+}
+
 char *getLine(char *buf, int size, FILE *f) {
   int c, i;
 
@@ -567,6 +698,30 @@ char *getLine(char *buf, int size, FILE *f) {
   return buf;
 }
 
+int gfseek(FILE *f, GFileOffset offset, int whence) {
+#if HAVE_FSEEKO
+  return fseeko(f, offset, whence);
+#elif HAVE_FSEEK64
+  return fseek64(f, offset, whence);
+#elif HAVE_FSEEKI64
+  return _fseeki64(f, offset, whence);
+#else
+  return fseek(f, offset, whence);
+#endif
+}
+
+GFileOffset gftell(FILE *f) {
+#if HAVE_FSEEKO
+  return ftello(f);
+#elif HAVE_FSEEK64
+  return ftell64(f);
+#elif HAVE_FSEEKI64
+  return _ftelli64(f);
+#else
+  return ftell(f);
+#endif
+}
+
 //------------------------------------------------------------------------
 // GDir and GDirEntry
 //------------------------------------------------------------------------
@@ -574,7 +729,7 @@ char *getLine(char *buf, int size, FILE *f) {
 GDirEntry::GDirEntry(char *dirPath, char *nameA, GBool doStat) {
 #ifdef VMS
   char *p;
-#elif defined(WIN32)
+#elif defined(_WIN32)
   int fa;
   GString *s;
 #elif defined(ACORN)
@@ -594,8 +749,8 @@ GDirEntry::GDirEntry(char *dirPath, char *nameA, GBool doStat) {
 #else
     s = new GString(dirPath);
     appendToPath(s, nameA);
-#ifdef WIN32
-    fa = GetFileAttributes(s->getCString());
+#ifdef _WIN32
+    fa = GetFileAttributesA(s->getCString());
     dir = (fa != 0xFFFFFFFF && (fa & FILE_ATTRIBUTE_DIRECTORY));
 #else
     if (stat(s->getCString(), &st) == 0)
@@ -613,15 +768,16 @@ GDirEntry::~GDirEntry() {
 GDir::GDir(char *name, GBool doStatA) {
   path = new GString(name);
   doStat = doStatA;
-#if defined(WIN32)
+#if defined(_WIN32)
   GString *tmp;
 
   tmp = path->copy();
   tmp->append("/*.*");
-  hnd = FindFirstFile(tmp->getCString(), &ffd);
+  hnd = FindFirstFileA(tmp->getCString(), &ffd);
   delete tmp;
 #elif defined(ACORN)
 #elif defined(MACOS)
+#elif defined(ANDROID)
 #else
   dir = opendir(name);
 #ifdef VMS
@@ -632,13 +788,14 @@ GDir::GDir(char *name, GBool doStatA) {
 
 GDir::~GDir() {
   delete path;
-#if defined(WIN32)
+#if defined(_WIN32)
   if (hnd) {
     FindClose(hnd);
     hnd = NULL;
   }
 #elif defined(ACORN)
 #elif defined(MACOS)
+#elif defined(ANDROID)
 #else
   if (dir)
     closedir(dir);
@@ -646,50 +803,65 @@ GDir::~GDir() {
 }
 
 GDirEntry *GDir::getNextEntry() {
-  struct dirent *ent;
   GDirEntry *e;
 
-  e = NULL;
-#if defined(WIN32)
-  e = new GDirEntry(path->getCString(), ffd.cFileName, doStat);
-  if (hnd  && !FindNextFile(hnd, &ffd)) {
-    FindClose(hnd);
-    hnd = NULL;
+#if defined(_WIN32)
+  if (hnd) {
+    e = new GDirEntry(path->getCString(), ffd.cFileName, doStat);
+    if (hnd  && !FindNextFileA(hnd, &ffd)) {
+      FindClose(hnd);
+      hnd = NULL;
+    }
+  } else {
+    e = NULL;
   }
 #elif defined(ACORN)
 #elif defined(MACOS)
-#else
+#elif defined(ANDROID)
+#elif defined(VMS)
+  struct dirent *ent;
+  e = NULL;
   if (dir) {
-#ifdef VMS
     if (needParent) {
       e = new GDirEntry(path->getCString(), "-", doStat);
       needParent = gFalse;
       return e;
     }
-#endif
     ent = readdir(dir);
-#ifndef VMS
-    if (ent && !strcmp(ent->d_name, "."))
-      ent = readdir(dir);
-#endif
-    if (ent)
+    if (ent) {
       e = new GDirEntry(path->getCString(), ent->d_name, doStat);
+    }
+  }
+#else
+  struct dirent *ent;
+  e = NULL;
+  if (dir) {
+    ent = (struct dirent *)readdir(dir);
+    if (ent && !strcmp(ent->d_name, ".")) {
+      ent = (struct dirent *)readdir(dir);
+    }
+    if (ent) {
+      e = new GDirEntry(path->getCString(), ent->d_name, doStat);
+    }
   }
 #endif
+
   return e;
 }
 
 void GDir::rewind() {
-#ifdef WIN32
+#ifdef _WIN32
   GString *tmp;
 
   if (hnd)
     FindClose(hnd);
   tmp = path->copy();
   tmp->append("/*.*");
-  hnd = FindFirstFile(tmp->getCString(), &ffd);
+  hnd = FindFirstFileA(tmp->getCString(), &ffd);
+  delete tmp;
 #elif defined(ACORN)
 #elif defined(MACOS)
+#elif defined(ANDROID)
 #else
   if (dir)
     rewinddir(dir);
