@@ -17,6 +17,7 @@
 
 #include <stdio.h>
 #include "gtypes.h"
+#include "gfile.h"
 #include "Object.h"
 
 class BaseStream;
@@ -97,13 +98,18 @@ public:
   // Get next line from stream.
   virtual char *getLine(char *buf, int size);
 
+  // Discard the next <n> bytes from stream.  Returns the number of
+  // bytes discarded, which will be less than <n> only if EOF is
+  // reached.
+  virtual Guint discardChars(Guint n);
+
   // Get current position in file.
-  virtual int getPos() = 0;
+  virtual GFileOffset getPos() = 0;
 
   // Go to a position in the stream.  If <dir> is negative, the
   // position is from the end of the file; otherwise the position is
   // from the start of the file.
-  virtual void setPos(Guint pos, int dir = 0) = 0;
+  virtual void setPos(GFileOffset pos, int dir = 0) = 0;
 
   // Get PostScript command for the filter(s).
   virtual GString *getPSFilter(int psLevel, const char *indent);
@@ -133,11 +139,11 @@ public:
 
   // Add filters to this stream according to the parameters in <dict>.
   // Returns the new stream.
-  Stream *addFilters(Object *dict);
+  Stream *addFilters(Object *dict, int recursion = 0);
 
 private:
 
-  Stream *makeFilter(char *name, Stream *str, Object *params);
+  Stream *makeFilter(char *name, Stream *str, Object *params, int recursion);
 
   int ref;			// reference count
 };
@@ -153,9 +159,9 @@ public:
 
   BaseStream(Object *dictA);
   virtual ~BaseStream();
-  virtual Stream *makeSubStream(Guint start, GBool limited,
-				Guint length, Object *dict) = 0;
-  virtual void setPos(Guint pos, int dir = 0) = 0;
+  virtual Stream *makeSubStream(GFileOffset start, GBool limited,
+				GFileOffset length, Object *dict) = 0;
+  virtual void setPos(GFileOffset pos, int dir = 0) = 0;
   virtual GBool isBinary(GBool last = gTrue) { return last; }
   virtual BaseStream *getBaseStream() { return this; }
   virtual Stream *getUndecodedStream() { return this; }
@@ -163,7 +169,7 @@ public:
   virtual GString *getFileName() { return NULL; }
 
   // Get/set position of first byte of stream within the file.
-  virtual Guint getStart() = 0;
+  virtual GFileOffset getStart() = 0;
   virtual void moveStart(int delta) = 0;
 
 private:
@@ -183,8 +189,8 @@ public:
   FilterStream(Stream *strA);
   virtual ~FilterStream();
   virtual void close();
-  virtual int getPos() { return str->getPos(); }
-  virtual void setPos(Guint pos, int dir = 0);
+  virtual GFileOffset getPos() { return str->getPos(); }
+  virtual void setPos(GFileOffset pos, int dir = 0);
   virtual BaseStream *getBaseStream() { return str->getBaseStream(); }
   virtual Stream *getUndecodedStream() { return str->getUndecodedStream(); }
   virtual Dict *getDict() { return str->getDict(); }
@@ -212,6 +218,9 @@ public:
   // Reset the stream.
   void reset();
 
+  // Close down the stream.
+  void close();
+
   // Gets the next pixel from the stream.  <pix> should be able to hold
   // at least nComps elements.  Returns false at end of file.
   GBool getPixel(Guchar *pix);
@@ -252,6 +261,8 @@ public:
 
   GBool isOk() { return ok; }
 
+  void reset();
+
   int lookChar();
   int getChar();
   int getBlock(char *blk, int size);
@@ -282,11 +293,11 @@ private:
 class FileStream: public BaseStream {
 public:
 
-  FileStream(FILE *fA, Guint startA, GBool limitedA,
-	     Guint lengthA, Object *dictA);
+  FileStream(FILE *fA, GFileOffset startA, GBool limitedA,
+	     GFileOffset lengthA, Object *dictA);
   virtual ~FileStream();
-  virtual Stream *makeSubStream(Guint startA, GBool limitedA,
-				Guint lengthA, Object *dictA);
+  virtual Stream *makeSubStream(GFileOffset startA, GBool limitedA,
+				GFileOffset lengthA, Object *dictA);
   virtual StreamKind getKind() { return strFile; }
   virtual void reset();
   virtual void close();
@@ -295,9 +306,9 @@ public:
   virtual int lookChar()
     { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
   virtual int getBlock(char *blk, int size);
-  virtual int getPos() { return bufPos + (int)(bufPtr - buf); }
-  virtual void setPos(Guint pos, int dir = 0);
-  virtual Guint getStart() { return start; }
+  virtual GFileOffset getPos() { return bufPos + (int)(bufPtr - buf); }
+  virtual void setPos(GFileOffset pos, int dir = 0);
+  virtual GFileOffset getStart() { return start; }
   virtual void moveStart(int delta);
 
 private:
@@ -305,14 +316,14 @@ private:
   GBool fillBuf();
 
   FILE *f;
-  Guint start;
+  GFileOffset start;
   GBool limited;
-  Guint length;
+  GFileOffset length;
   char buf[fileStreamBufSize];
   char *bufPtr;
   char *bufEnd;
-  Guint bufPos;
-  int savePos;
+  GFileOffset bufPos;
+  GFileOffset savePos;
   GBool saved;
 };
 
@@ -325,8 +336,8 @@ public:
 
   MemStream(char *bufA, Guint startA, Guint lengthA, Object *dictA);
   virtual ~MemStream();
-  virtual Stream *makeSubStream(Guint start, GBool limited,
-				Guint lengthA, Object *dictA);
+  virtual Stream *makeSubStream(GFileOffset start, GBool limited,
+				GFileOffset lengthA, Object *dictA);
   virtual StreamKind getKind() { return strWeird; }
   virtual void reset();
   virtual void close();
@@ -335,9 +346,9 @@ public:
   virtual int lookChar()
     { return (bufPtr < bufEnd) ? (*bufPtr & 0xff) : EOF; }
   virtual int getBlock(char *blk, int size);
-  virtual int getPos() { return (int)(bufPtr - buf); }
-  virtual void setPos(Guint pos, int dir = 0);
-  virtual Guint getStart() { return start; }
+  virtual GFileOffset getPos() { return (GFileOffset)(bufPtr - buf); }
+  virtual void setPos(GFileOffset pos, int dir = 0);
+  virtual GFileOffset getStart() { return start; }
   virtual void moveStart(int delta);
 
 private:
@@ -363,25 +374,25 @@ private:
 class EmbedStream: public BaseStream {
 public:
 
-  EmbedStream(Stream *strA, Object *dictA, GBool limitedA, Guint lengthA);
+  EmbedStream(Stream *strA, Object *dictA, GBool limitedA, GFileOffset lengthA);
   virtual ~EmbedStream();
-  virtual Stream *makeSubStream(Guint start, GBool limitedA,
-				Guint lengthA, Object *dictA);
+  virtual Stream *makeSubStream(GFileOffset start, GBool limitedA,
+				GFileOffset lengthA, Object *dictA);
   virtual StreamKind getKind() { return str->getKind(); }
   virtual void reset() {}
   virtual int getChar();
   virtual int lookChar();
   virtual int getBlock(char *blk, int size);
-  virtual int getPos() { return str->getPos(); }
-  virtual void setPos(Guint pos, int dir = 0);
-  virtual Guint getStart();
+  virtual GFileOffset getPos() { return str->getPos(); }
+  virtual void setPos(GFileOffset pos, int dir = 0);
+  virtual GFileOffset getStart();
   virtual void moveStart(int delta);
 
 private:
 
   Stream *str;
   GBool limited;
-  Guint length;
+  GFileOffset length;
 };
 
 //------------------------------------------------------------------------
@@ -623,9 +634,11 @@ private:
   DCTHuffTable acHuffTables[4];	// AC Huffman tables
   int numDCHuffTables;		// number of DC Huffman tables
   int numACHuffTables;		// number of AC Huffman tables
-  Guchar *rowBuf[4][32];	// buffer for one MCU (non-progressive mode)
+  Guchar *rowBuf;
+  Guchar *rowBufPtr;		// current position within rowBuf
+  Guchar *rowBufEnd;		// end of valid data in rowBuf
   int *frameBuf[4];		// buffer for frame (progressive mode)
-  int comp, x, y, dy;		// current position within image/MCU
+  int comp, x, y;		// current position within image/MCU
   int restartCtr;		// MCUs left until restart
   int restartMarker;		// next restart marker
   int eobRun;			// number of EOBs left in the current run
@@ -902,4 +915,42 @@ private:
   GBool fillBuf();
 };
 
+//------------------------------------------------------------------------
+// LZWEncoder
+//------------------------------------------------------------------------
+
+struct LZWEncoderNode {
+  int byte;
+  LZWEncoderNode *next;		// next sibling
+  LZWEncoderNode *children;	// first child
+};
+
+class LZWEncoder: public FilterStream {
+public:
+
+  LZWEncoder(Stream *strA);
+  virtual ~LZWEncoder();
+  virtual StreamKind getKind() { return strWeird; }
+  virtual void reset();
+  virtual int getChar();
+  virtual int lookChar();
+  virtual GString *getPSFilter(int psLevel, const char *indent)
+    { return NULL; }
+  virtual GBool isBinary(GBool last = gTrue) { return gTrue; }
+  virtual GBool isEncoder() { return gTrue; }
+
+private:
+
+  LZWEncoderNode table[4096];
+  int nextSeq;
+  int codeLen;
+  Guchar inBuf[4096];
+  int inBufLen;
+  int outBuf;
+  int outBufLen;
+  GBool needEOD;
+
+  void fillBuf();
+};
+
 #endif
