@@ -120,7 +120,8 @@ int main(int argc, char *argv[]) {
 
   // check for copy permission
   if (!doc->okToCopy()) {
-    error(-1, "Copying of images from this document is not allowed.");
+    error(errNotAllowed, -1,
+	  "Copying of images from this document is not allowed.");
     exitCode = 3;
     goto err1;
   }
@@ -134,7 +135,8 @@ int main(int argc, char *argv[]) {
   // write image files
   imgOut = new ImageOutputDev(imgRoot, dumpJPEG);
   if (imgOut->isOk()) {
-    doc->displayPages(imgOut, firstPage, lastPage, 72, 72, 0, gTrue, gFalse);
+    doc->displayPages(imgOut, firstPage, lastPage, 72, 72, 0,
+		      gFalse, gTrue, gFalse);
   }
   delete imgOut;
 
